from __future__ import absolute_import, print_function, unicode_literals

try:
    input = raw_input
except NameError:
    pass

import sys
import time

import parse

from .api import API
from .wrapper import Browser, Network


class Client(object):
    """
    Pocket API Access Client
    """

    def __init__(self, credentials, args):
        self._args = args
        self._credentials = credentials
        self._api_endpoint = ''
        self._payload = {}
        self._req = None

        self._format_spec = ''
        self._unformat_spec = None
        self._output = []
        self._input = []

    def _api_request(self):
        # add API access credentials
        self._payload.update(self._credentials)

        # access API
        self._req = Network.post_request(self._api_endpoint, self._payload)

    def _output_to_file(self):
        with open(self._args.output, 'w+') as outfile:
            for info in self._output:
                line = self._format_spec.format(**info)
                try:
                    outfile.write(line)
                except UnicodeEncodeError:
                    outfile.write(line.encode('utf-8'))

    def _print_to_console(self):
        for info in self._output:
            line = self._format_spec.format(**info)
            try:
                print(line, end='')
            except UnicodeEncodeError:
                print(line.encode('utf-8'), end='')

    def _open_in_browser(self):
        # open a new window
        Browser.open_new_window(self._output[0]['link'])

        for info in self._output[1:]:
            # open new tabs
            Browser.open_new_tab(info['link'])
            # pace webbrowser actions
            time.sleep(2)

    def _get_console_input(self):
        print('Enter data: {0}'.format(self._args.format.strip()))
        try:
            while True:
                data = input().strip()
                if data:
                    info = self._unformat_spec.parse(data)
                    self._input.append(info)
                else:
                    raise EOFError
        except EOFError:
            pass

    def _get_redirect_input(self):
        for line in sys.stdin.readlines():
            data = line.strip()
            if data:
                info = self._unformat_spec.parse(data)
                self._input.append(info)
            else:
                continue

    def _get_file_input(self):
        with open(self._args.input, 'r') as f:
            for line in f.readlines():
                data = line.strip()
                if data:
                    info = self._unformat_spec.parse(data)
                    self._input.append(info)
                else:
                    continue

    def _validate_format(self):
        # interpret escape sequences
        try:
            self._args.format = bytes(
                self._args.format, 'utf-8'
            ).decode('unicode_escape')
        except TypeError:
            self._args.format = self._args \
                .format.decode('unicode_escape')

        info = dict((key, None) for key in API.INFO_KEYS)

        try:
            self._args.format.format(**info)
        except KeyError:
            print('Invalid Format Specifier !')
            sys.exit(1)
        else:
            self._format_spec = self._args.format + '\n'
            self._unformat_spec = parse.compile(self._args.format)

    def _get(self):
        # create request payload
        payload = {
            'state': self._args.state,
            'sort': self._args.sort,
        }

        if self._args.content != 'all':
            payload['contentType'] = self._args.content

        if self._args.count != -1:
            payload['count'] = self._args.count

        if self._args.query:
            payload['search'] = self._args.query

        if self._args.tag == '-1':
            pass
        elif self._args.tag == '0':
            payload['tag'] = '_untagged_'
        else:
            payload['tag'] = self._args.tag

        if self._args.favorite != -1:
            payload['favorite'] = self._args.favorite

        if self._args.domain:
            payload['domain'] = self._args.domain

        self._payload = payload
        self._api_endpoint = API.RETRIEVE_URL

        self._api_request()

        json_data = self._req.api_json
        items = json_data.get('list') or {}

        if len(items) == 0:
            print('No items found !')
            sys.exit(0)

        self._output = [
            {
                'id': item.get('item_id'),
                'title': item.get('resolved_title'),
                'link': item.get('resolved_url'),
                'excerpt': item.get('excerpt'),
                'tags': item.get('tags'),
            }
            for item in items.values()
        ]

    def _put(self):
        payload = {
            'actions': [
                {
                    'action': 'add',
                    'url': info['link'],
                }
                for info in self._input
            ]
        }

        self._payload = payload
        self._api_endpoint = API.MODIFY_URL

        self._api_request()

    def _modify(self):
        if self._args.delete:
            action = 'delete'
        elif self._args.archive != -1:
            if self._args.archive == 1:
                action = 'archive'
            else:
                action = 'readd'
        elif self._args.favorite != -1:
            if self._args.favorite == 1:
                action = 'favorite'
            else:
                action = 'unfavorite'
        else:
            action = ''

        payload = {
            'actions': [
                {
                    'action': action,
                    'item_id': info['id'],
                }
                for info in self._input
            ]
        }

        self._payload = payload
        self._api_endpoint = API.MODIFY_URL

        self._api_request()

    def run(self):

        # validate format specifier
        self._validate_format()

        if self._args.do == 'get':
            self._get()

            # print output
            self._print_to_console()

            # redirect output
            if self._args.output == 'browser':
                self._open_in_browser()
            elif self._args.output:
                self._output_to_file()

        else:
            if self._args.input == 'console':
                self._get_console_input()
            elif self._args.input == 'redirect':
                self._get_redirect_input()
            else:
                self._get_file_input()

            if self._args.do == 'put':
                self._put()
            elif self._args.do == 'mod':
                self._modify()



from __future__ import absolute_import, print_function, unicode_literals

import os
import sys
import traceback
import webbrowser
from collections import OrderedDict

import requests

from .api import API


def print_bug_report():
    """
    Prints a usable bug report
    """

    separator = '-' * 66
    pretty_print = lambda s:\
        print(separator + '\n{0}\n'.format(s) + separator)

    python_version = str(sys.version_info[:3])
    arguments = str(sys.argv[1:])
    try:
        import pip
    except ImportError:
        packages = '`pip` not installed !'
    else:
        packages = str([
            (package.key, package.version)
            for package in pip.get_installed_distributions()
        ])

    pretty_print('Bug Report :')
    print('Python Version : ' + python_version)
    print('Installed Packages : \n' + packages)
    print('Runtime Arguments : \n' + arguments)
    pretty_print('Error Traceback :')
    traceback.print_exc()
    pretty_print(
        '`pockyt` encountered an error ! Submit the (above) '
        'bug report at :\n` {0} `'.format(API.ISSUE_URL)
    )


class SuppressedStdout(object):
    """
    Suppresses STDOUT, utilize with the `with` keyword
    """

    def __init__(self):
        self.orig_stdout = sys.stdout.fileno()
        self.copy_stdout = os.dup(self.orig_stdout)
        self.devnull = None

    def __enter__(self):
        self.devnull = open(os.devnull, 'w')
        os.dup2(self.devnull.fileno(), self.orig_stdout)

    def __exit__(self, *args):
        os.dup2(self.copy_stdout, self.orig_stdout)
        self.devnull.close()


class Network(object):
    """
    Safe POST Request, with error-handling
    """

    @staticmethod
    def post_request(link, payload):
        req = requests.post(link, json=payload)

        if req.status_code != 200:
            print('Error {0} ! : {1}'.format(
                req.headers.get('X-Error-Code'),
                req.headers.get('X-Error'),
            ))
            sys.exit(1)
        else:
            try:  # preserve json response ordering, as per API
                req.api_json = req.json(object_pairs_hook=OrderedDict)
            except ValueError:
                req.api_json = {}
            finally:
                return req


class Browser(object):
    """
    Silently open browser windows/tabs
    """

    @staticmethod
    def open(link, new=0, autoraise=True):
        with SuppressedStdout():
            webbrowser.open(
                url=link,
                new=new,
                autoraise=autoraise,
            )

    @classmethod
    def open_new_window(cls, link):
        cls.open(link, new=1)

    @classmethod
    def open_new_tab(cls, link):
        cls.open(link, new=2)



from __future__ import unicode_literals


class API(object):
    """
    Pocket API Access Data
    """

    APP_NAME = 'pockyt'

    POCKET_URL = 'https://getpocket.com/'
    API_URL = POCKET_URL + 'v3/'

    REDIRECT_URL = 'https://www.github.com/arvindch/{0}'.format(APP_NAME)
    ISSUE_URL = REDIRECT_URL + '/issues/new'
    APP_CREATE_URL = 'http://getpocket.com/developer/apps/new'

    REQUEST_TOKEN_URL = API_URL + 'oauth/request'
    ACCESS_TOKEN_URL = API_URL + 'oauth/authorize'

    RETRIEVE_URL = API_URL + 'get'
    MODIFY_URL = API_URL + 'send'

    CONFIG_HEADER = 'CREDENTIALS'

    INFO_KEYS = ['id', 'title', 'link', 'excerpt', 'tags']

    @classmethod
    def get_auth_user_url(cls, rt):
        return cls.POCKET_URL + 'auth/authorize?request_token={0}' \
                                '&redirect_uri={1}'.format(rt, cls.REDIRECT_URL)



from __future__ import absolute_import, print_function

import sys
from setuptools import setup


if sys.version_info[0:2] not in [(2, 7), (3, 4)]:
    print('This version of Python is unsupported !\n'
          'Please use Python 2.7.x or 3.4.x !')
    sys.exit(1)


name = 'pockyt'
version = '0.7'


setup(
    name=name,
    packages=[name],
    version=version,
    description='automate and manage your pocket collection',
    long_description=open('README.rst').read(),
    author='Arvind Chembarpu',
    author_email='achembarpu@gmail.com',
    url='https://github.com/arvindch/{0}'.format(name),
    license='GPLv3+',
    install_requires=[
        'requests>=2.4',
        'parse>=1.6',
    ],
    download_url='https://github.com/arvindch/{0}/tarball/{1}'.format(name, version),
    keywords=['pocket', 'commandline', 'automation'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Topic :: Utilities',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
    ],
    entry_points={
        'console_scripts': [
            'pockyt=pockyt.pockyt:main',
        ],
    },
)



from __future__ import absolute_import, unicode_literals

import argparse
import sys

from .api import API
from .auth import Authenticator
from .client import Client
from .wrapper import print_bug_report


class Pockyt(object):
    """
    pockyt - Pocket Commandline Client
    """

    def __init__(self):
        self._setup_parsers()
        self._args = self.parser.parse_args()

    def _setup_parsers(self):
        # main parser
        self.parser = argparse.ArgumentParser(
            description='automate and manage your pocket collection',
            epilog='Project Page: ` {0} `'.format(API.REDIRECT_URL),
        )

        # subcommand parsers
        subparsers = self.parser.add_subparsers(dest='do')

        # connect account
        subparsers.add_parser('reg', help='connect a pocket account')

        # get items from collection
        get_parser = subparsers.add_parser(
            'get', help='get pocket collection, with useful item_info',
        )
        get_parser.add_argument(
            '-c', '--content', default='all', metavar='<type>',
            choices=['all', 'article', 'video', 'image'],
            help='content type : <type> : {all, [article, video, image]}',
        )
        get_parser.add_argument(
            '-s', '--state', default='all', metavar='<state>',
            choices=['all', 'unread', 'archive'],
            help='collection state : <state> : {all, [unread, archive]}',
        )
        get_parser.add_argument(
            '-r', '--sort', default='newest', metavar='<order>',
            choices=['newest', 'oldest', 'title', 'site'],
            help='item sorting : <order> : {newest, [oldest, title, site]}',
        )
        get_parser.add_argument(
            '-n', '--count', metavar='<amount>', default=-1, type=int,
            help='number of items : <amount> : {-1: all, [n: amount]}',
        )
        get_parser.add_argument(
            '-q', '--query', metavar='<query>',
            help='search query : <query> : {None}',
        )
        get_parser.add_argument(
            '-t', '--tag', metavar='<option>',
            default='-1', type=str,
            help='filter tag : {-1: nofilter, '
                 '[tagname: tagged, 0: untagged}',
        )
        get_parser.add_argument(
            '-v', '--favorite', metavar='<option>',
            default=-1, type=int, choices=[-1, 0, 1],
            help='filter favorites : <option> : {-1: No Filter, '
                 '[1: favorited, 0: un-favorited]}',
        )
        get_parser.add_argument(
            '-d', '--domain', metavar='<domain>',
            help='restrict items to domain : <domain> : {None}',
        )
        get_parser.add_argument(
            '-f', '--format', metavar='<specifier>',
            default='{id} | {title} | {link}',
            help='format output : <specifier> : {\'{id} | {title} | {link}\', '
                 '[id, title, link, excerpt, tags]}',
        )
        get_parser.add_argument(
            '-o', '--output', metavar='<option>',
            help='redirect output : <option> : {None, [browser, filename]}',
        )

        # add items to collection
        put_parser = subparsers.add_parser(
            'put', help='add to pocket collection, using links',
        )
        put_parser.add_argument(
            '-f', '--format', metavar='<specifier>', default='{link}',
            help='unformat input : <specifier> : {\'{link}\', '
                 '[id, title, link, excerpt, tags]}',
        )
        put_parser.add_argument(
            '-i', '--input', default='console', metavar='<option>',
            help='obtain input : <option> : {console, [redirect, filename]}',
        )

        # modify items in collection
        mod_parser = subparsers.add_parser(
            'mod', help='modify pocket collection, using item_ids',
        )
        mod_parser.add_argument(
            '-f', '--format', metavar='<specifier>', default='{id}',
            help='unformat input : <specifier> : {\'{id}\', '
                 '[id, title, link, excerpt, tags]}',
        )
        mod_parser.add_argument(
            '-i', '--input', default='console', metavar='<option>',
            help='obtain input : <option> : {console, [redirect, filename]}',
        )
        options = mod_parser.add_mutually_exclusive_group(required=True)
        options.add_argument(
            '-d', '--delete', action='store_true', help='delete items',
        )
        options.add_argument(
            '-a', '--archive', default=-1, metavar='<option>',
            choices=[-1, 0, 1], type=int,
            help='archive items : <option> : {-1: None, [1: archive, 0: unarchive]}',
        )
        options.add_argument(
            '-v', '--favorite', default=-1, metavar='<option>',
            choices=[-1, 0, 1], type=int,
            help='favorite items : <option> : {-1: None, [1: favorite, 0: unfavorite]}',
        )

    def run(self):
        if not self._args.do:
            # print usage help
            self.parser.print_help()
            sys.exit(1)

        # setup authentication
        auth = Authenticator(self._args)
        auth.run()

        # start pocket interaction
        pocket = Client(auth.credentials, self._args)
        pocket.run()


def main():
    try:
        app = Pockyt()
        app.run()
    except KeyboardInterrupt:
        sys.exit(1)
    except Exception:
        print_bug_report()
        sys.exit(1)


if __name__ == '__main__':
    main()



from __future__ import absolute_import, print_function, unicode_literals

try:
    input = raw_input
except NameError:
    pass

try:
    import ConfigParser as configparser
except ImportError:
    import configparser
try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse

import os
import sys

from .api import API
from .wrapper import Network


class Authenticator(object):
    """
    Pocket API Access Authenticator
    """

    def __init__(self, args):
        self._args = args
        self._config = None
        self._consumer_key = None
        self._request_token = None
        self._access_token = None
        self._username = None

        self._config_path = os.path.join(
            os.path.expanduser('~'),
            '.{0}'.format(API.APP_NAME),
        )

    @property
    def credentials(self):
        return {
            'consumer_key': self._consumer_key,
            'access_token': self._access_token,
            'username': self._username,
        }

    def _save_config(self):
        self._config = configparser.ConfigParser()
        self._config.add_section(API.CONFIG_HEADER)
        for (k, v) in self.credentials.items():
            self._config.set(API.CONFIG_HEADER,  k, v)

        with open(self._config_path, 'w+') as cf:
            self._config.write(cf)

    def _load_config(self):
        self._config = configparser.ConfigParser()
        self._config.read(self._config_path)

    def _obtain_request_token(self):
        payload = {
            'consumer_key': self._consumer_key,
            'redirect_uri': API.REDIRECT_URL,
        }
        req = Network.post_request(API.REQUEST_TOKEN_URL, payload)
        info = urlparse.parse_qs(req.text)

        self._request_token = info['code'][0]

    def _obtain_access_token(self):
        payload = {
            'consumer_key': self._consumer_key,
            'code': self._request_token,
        }
        req = Network.post_request(API.ACCESS_TOKEN_URL, payload)
        info = urlparse.parse_qs(req.text)

        self._access_token = info['access_token'][0]
        self._username = info['username'][0]

    def _setup(self):
        input('Create an application, via this link :\n` {0} `\n'
              'Press Enter when done...'
              .format(API.APP_CREATE_URL))

        self._consumer_key = input('Enter your Consumer Key: ')

        self._obtain_request_token()

        input('Connect an account, via this link :\n` {0} `\n'
              'Press Enter to when done...'
              .format(API.get_auth_user_url(self._request_token)))

        self._obtain_access_token()

        self._save_config()

        print('Welcome {0} !\n{1} is now ready for usage !'
              .format(self._username, API.APP_NAME))

    def _load(self):
        self._load_config()
        try:
            self._consumer_key = self._config.get(API.CONFIG_HEADER, 'consumer_key')
            self._access_token = self._config.get(API.CONFIG_HEADER, 'access_token')
            self._username = self._config.get(API.CONFIG_HEADER, 'username')
        except KeyError:
            print('Connect an account first!')
            sys.exit(1)

    def run(self):
        if self._args.do == 'reg':
            self._setup()
            sys.exit(0)
        else:
            self._load()
            return
